/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

This file programs the output LUT:


*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <math.h>

/*******************************************************************/

void  FalseColor    (void);
void  Inverse       (void);
void  Standard      (void);
void  Overlay       (void);

/*******************************************************************/

void main()
  {
  vmode(vmLive); 
  shutter(10000);
  set_ovlmask(0);

  pstr("False Color    Display (Press any key)\n");
  FalseColor();
  rs232rcv();

  pstr("Inverse        Display (Press any key)\n");
  Inverse();
  rs232rcv();

  pstr("Standard       Display (Press any key)\n");
  Standard();
  rs232rcv();

  pstr("Overlay        Display (Press any key)\n");
  pstr("Ovelay and Translucen\n");
  Overlay();
  rs232rcv();
		
  pstr("Select Overlay Colors  (Press any key)\n");
  set_ovlmask(246);
  rs232rcv();

  pstr("Clear Overlay\n");
  OvlClearAll;
  }

/*******************************************************************/

void Standard (void)
  {
  init_LUT();
  }

/*******************************************************************/

void Inverse (void)
  {
  int i;
  volatile unsigned int *lut;

  lut = (volatile unsigned int *)VIRTX_CLUT;

  for(i=255; i>=0 ;i--)
    {				 
    *lut++ = 
           (i     ) +  /* RED   */
           (i << 8) +  /* GREEN */
           (i <<16) ;  /* BLUE  */
	}
  }

/*******************************************************************/

void FalseColor (void)
  {
  int i;
  volatile unsigned int *lut;

  lut = (volatile unsigned int *)VIRTX_CLUT;

  /* grey values from 0-15 become color 1 */
  for(i=0; i<16 ;i++)
    {				 
    *lut++ = 
           (210     ) +  /* RED   */
           (255 << 8) +  /* GREEN */
           (  0 <<16) ;  /* BLUE  */
	}

  /* grey values from 16-31 become color 2 */
  for(; i<32 ;i++)
    {				 
    *lut++ = 
           (140     ) +  /* RED   */
           (255 << 8) +  /* GREEN */
           (  0 <<16) ;  /* BLUE  */
	}

  /* grey values from 32-47 become color 3 */
  for(; i<48 ;i++)
    {				 
    *lut++ = 
           (  0     ) +  /* RED   */
           (255 << 8) +  /* GREEN */
           ( 10 <<16) ;  /* BLUE  */
	}

  /* grey values from 48-63 become color 4 */
  for(; i<64 ;i++)
    {				 
    *lut++ = 
           (  0     ) +  /* RED   */
           (255 << 8) +  /* GREEN */
           (140 <<16) ;  /* BLUE  */
	}

  /* grey values from 64-79 become color 5 */
  for(; i<80 ;i++)
    {				 
    *lut++ = 
           (  0     ) +  /* RED   */
           (255 << 8) +  /* GREEN */
           (180 <<16) ;  /* BLUE  */
	}

  /* grey values from 80-95 become color 6 */
  for(; i<96 ;i++)
    {				 
    *lut++ = 
           (  0     ) +  /* RED   */
           (240 << 8) +  /* GREEN */
           (255 <<16) ;  /* BLUE  */
	}

  /* grey values from 96-111 become color 7 */
  for(; i<112 ;i++)
    {				 
    *lut++ = 
           (  0     ) +  /* RED   */
           (210 << 8) +  /* GREEN */
           (255 <<16) ;  /* BLUE  */
	}

  /* grey values from 112-127 become color 8 */
  for(; i<128 ;i++)
    {				 
    *lut++ = 
           (  0     ) +  /* RED   */
           ( 70 << 8) +  /* GREEN */
           (255 <<16) ;  /* BLUE  */
	}

  /* grey values from 128-143 become color 9 */
  for(; i<144 ;i++)
    {				 
    *lut++ = 
           (  0     ) +  /* RED   */
           (  0 << 8) +  /* GREEN */
           (255 <<16) ;  /* BLUE  */
	}

  /* grey values from 144-159 become color 10 */
  for(; i<160 ;i++)
    {				 
    *lut++ = 
           (160     ) +  /* RED   */
           (  0 << 8) +  /* GREEN */
           (255 <<16) ;  /* BLUE  */
	}

  /* grey values from 160-175 become color 11 */
  for(; i<176 ;i++)
    {				 
    *lut++ = 
           (255     ) +  /* RED   */
           (  0 << 8) +  /* GREEN */
           (140 <<16) ;  /* BLUE  */
	}

  /* grey values from 176-191 become color 12 */
  for(; i<192 ;i++)
    {				 
    *lut++ = 
           (255     ) +  /* RED   */
           (  8 << 8) +  /* GREEN */
           (  0 <<16) ;  /* BLUE  */
	}

  /* grey values from 192-207 become color 13 */
  for(; i<208 ;i++)
    {				 
    *lut++ = 
           (255     ) +  /* RED   */
           ( 60 << 8) +  /* GREEN */
           (  0 <<16) ;  /* BLUE  */
	}

  /* grey values from 208-223 become color 14 */
  for(; i<224 ;i++)
    {				 
    *lut++ = 
           (255     ) +  /* RED   */
           (128 << 8) +  /* GREEN */
           (  0 <<16) ;  /* BLUE  */
	}

  /* grey values from 224-239 become color 15 */
  for(; i<240 ;i++)
    {				 
    *lut++ = 
           (255     ) +  /* RED   */
           (200 << 8) +  /* GREEN */
           (  0 <<16) ;  /* BLUE  */
	}

  /* grey values from 240-255 become color 16 */
  for(; i<256 ;i++)
    {				 
    *lut++ = 
           (255     ) +  /* RED   */
           (255 << 8) +  /* GREEN */
           (  0 <<16) ;  /* BLUE  */
	}

  }

/*******************************************************************/

void Overlay (void)
  {
  image OvlCol1, OvlCol2, TraCol1, TraCol2;

  /* Working Page */
  OvlSetLogPage((int)OvlGetPhysPage);

  /* Clear Overlay */
  OvlClearAll;

  /* Define image variable for Overlay */
  ImageAssign(&OvlCol1, OvlBitAddr(1*OvlGetColumns/5, 1*OvlGetRows/5), OvlGetColumns/5, OvlGetRows/5, OvlGetPitch);  
  ImageAssign(&OvlCol2, OvlBitAddr(3*OvlGetColumns/5, 1*OvlGetRows/5), OvlGetColumns/5, OvlGetRows/5, OvlGetPitch);  
  ImageAssign(&TraCol1, OvlBitAddr(1*OvlGetColumns/5, 3*OvlGetRows/5), OvlGetColumns/5, OvlGetRows/5, OvlGetPitch);  
  ImageAssign(&TraCol2, OvlBitAddr(3*OvlGetColumns/5, 3*OvlGetRows/5), OvlGetColumns/5, OvlGetRows/5, OvlGetPitch);  

  /* Define Color */
  set_overlay_bit(2, 255,   0,   0); /* red */
  set_overlay_bit(3,   0, 255,   0); /* green */
  set_translucent(1,   0,   0, 255); /* blue translucent */
  set_translucent(2, 255, 255,   0); /* yellow translucent */

  set(&OvlCol1,   4);
  set(&OvlCol2,   8);
  set(&TraCol1,   1);
  set(&TraCol2,   2);

  OvlCol1.st-=(12L*(long)OvlGetPitch);
  OvlCol2.st-=(12L*(long)OvlGetPitch);
  TraCol1.st-=(12L*(long)OvlGetPitch);
  TraCol2.st-=(12L*(long)OvlGetPitch);

  chprint1("Red",&OvlCol1,1,1,4);
  chprint1("Green",&OvlCol2,1,1,8);
  chprint1("Blue",&TraCol1,1,1,1);
  chprint1("Yellow",&TraCol2,1,1,2);

  set_ovlmask(255);
  }

/*******************************************************************/

