/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

This file programs the input LUT in three differnt ways:
(new 10 bit input LUT for SPARTAN II devices)

1) inverse
2) logarithmic
3) linear
		   
Input  10 Bit (LUT 1009 values)
Output 8 Bit

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <math.h>

/*******************************************************************/

void program_ilut(unsigned int *table);

/*******************************************************************/

#define OFS 15		/*offeset for input grey level 0-15 that don't exist*/

/*******************************************************************/

void main()
  {
  int i;
  int table[1024];


  vmode(vmLive); 
  shutter(10000);


  pstr("Input LUT inverse     (Press any key)\n");
  for(i=0; i<1024; i++)
    {
    table[i] = (256 *  (1023-i) ) / (1024 - OFS);

    if (table[i]>255) table[i]=255;
    if (table[i]<  0) table[i]=0;
    }
  program_ilut((unsigned int *)table);
  rs232rcv();



  pstr("Input LUT logarithmic (Press any key)\n");
  for(i=  0; i< OFS; i++) 
    table[i]=0;
  for(i=OFS; i<1024; i++)
    {
    table[i]  = (int)(85.0*log10((float)(i-OFS+1))+0.5) ;
    }
  program_ilut((unsigned int *)table);
  rs232rcv();



  pstr("Input LUT linear\n");
  for(i=0; i<1024; i++)
    {
    table[i] = (256 * (i - OFS)) / (1024 - OFS);

    if (table[i]>255) table[i]=255;
    if (table[i]<  0) table[i]=0;
    }
  program_ilut((unsigned int *)table);

}

/*******************************************************************/

void program_ilut(unsigned int *table)
{
int i,x ;
volatile unsigned int *ilut  = (volatile unsigned int *)VIRTX_ILUT;

for(i=0; i<1024 ;i+=4)
  {
  x  =  table[i]   & 0xff;
  x |= (table[i+1] & 0xff) << 8;
  x |= (table[i+2] & 0xff) << 16;
  x |= (table[i+3] & 0xff) << 24;
  *ilut++ = x;
  }
}

/*******************************************************************/

