/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

The program is searching for black areas in a white background.
Black areas are darker than the treshold value. The program
gives the size, the position of every black area and shows the
position on the monitor.

!!! NEW: PROGRAM HAS BEEN UPDATED TO VCLIB 3.0 !!!

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

#define  EVEN(x)  ((x)&0xFFFE)

/*******************************************************************/

void blobtest     (int x, int y, int dx, int dy, int color);
int  find_objects (image *area, ftr *f, int maxobjects, int th, int display);
void cross_image  (int line, int column, int size, int color);

/*******************************************************************/

int ms;

/*******************************************************************/

void main(void)

  {   
  int   x, y, dx, dy;
  char  resp;

  dx = EVEN(640);
  dy = EVEN(480);
  x  = EVEN((ScrGetColumns-dx)/2);
  y  = EVEN((ScrGetRows   -dy)/2);

  /* Working Page */
  ScrSetLogPage((int)ScrGetPhysPage);

  vmode(vmLive);
  print("\nblob analyse ('w'=white 'b'=black objects)\n"); resp=rs232rcv();
  tpict();
  
  do
    {
    tpict();

    if (resp=='w') blobtest(x, y, dx, dy, -1);
    if (resp=='b') blobtest(x, y, dx, dy,  0);

    print("\nblob analyse (%dx%d): t=%dms",dx,dy,ms); 
    print("\n\nnew run? ('w'=white 'b'=black 'q'=quit)");  
    resp=rs232rcv();
    }
  while(resp !='q' );

  /* live picture at the end */
  vmode(vmLive);
  }

/*******************************************************************/


/*******************************************************************/
/*********************** SUBROUTINE ********************************/
/*******************************************************************/


/*******************************************************************/

#define MAXOBJECTS  100

void blobtest (int x, int y, int dx, int dy, int color)

  {
  int i, j=0, nobj,threshold=100,display=1;
  image area;
  ftr f[MAXOBJECTS];


  /* Search area is the whole image */
  ImageAssign(&area,ScrByteAddr(x,y),dx,dy,ScrGetPitch);
  
  /* feature extraction */
  nobj=find_objects(&area,f,MAXOBJECTS,threshold,display);
  
  print("\n\nFound %d white and black objects (-1 means more than %d objects)",nobj,MAXOBJECTS);

  if (color) print("\nwhite objects:");
  else       print("\nblack objects:");

  for(i=0;i<nobj;i++)
    {
    /* looking for color objects (black=0, white=-1) */
    if (f[i].color==color) 
      {
      /* print via seriell at 9600 Baud takes a long time */
      print("\nArea %d: Size=%d Center of Gravity = %d,%d",
               ++j,f[i].area,f[i].x_center+x,f[i].y_center+y);
      cross_image(EVEN((int)f[i].y_center+y),EVEN((int)f[i].x_center+x),10,127);
      }
    }
   }

/*******************************************************************/

int find_objects (image *a, ftr *f, int maxobjects, int th, int display)

  {
  long maxlng= 0x020000L;
  U16 *rlc;
  U16 *slc;
  int  objects;

  /* allocate DRAM Memory */
  rlc = (U16 *) sysmalloc((maxlng* sizeof(U16)+1) / sizeof(U32), MDATA);
  if (rlc == NULL) {pstr("DRAM Memory overrun\n"); return(-1);}

  /* create RLC */
  ms = getvar(MSEC);
  slc=rlcmk(a, th, rlc, maxlng);
  if (slc == NULL ) {pstr("RLC overrun\n"); sysfree(rlc); return(-1);}

  
  /* object labelling */
  if(sgmt(rlc,slc)==0L) {pstr("object number overrun\n"); sysfree(rlc); return(-1);}

  /* feature extraction */
  objects=rl_ftr2(rlc, f, maxobjects);

  ms = getvar(MSEC)-ms;
  if(ms<0) ms+=1000;
 
  /* display RLC */
  if (display) rlcout(a,rlc,0,255);

  /* free allocation */
  sysfree(rlc);  

  return(objects);
  }

/*******************************************************************/

void cross_image (int line, int column, int size, int color)

  {
  int i;

  for (i=-size;i<=size;i++)
    {
    wpix(color,ScrByteAddr(column  ,line+i));
    wpix(color,ScrByteAddr(column+i,line  ));
    }
  }

/*******************************************************************/

/*******************************************************************/
/*******************************************************************/
/*******************************************************************/
