/************************************************************************************************************************/
/*

Software for smart cameras from Vision Components
-------------------------------------------------

Program: Klaus Schneider, VC


Program Description:
--------------------


*/
/************************************************************************************************************************/

#include <math.h>

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

#include "flib.h"

/************************************************************************************************************************/

I16  VerticalEdgeDetection       (image *EdgeArea, image *Brightness, image *Difference);
void DisplayGraph                (image *Graph, U32 *values, I16 nr);
void IniDisplay                  (void);

/************************************************************************************************************************/

void main(void)
{
  I16   response; 
  I16   x, y, dx, dy, pitch;
  image EdgeAreaHor, Brightness, Difference, Area;
  
  // ini variables
    
  // ini working page
  pitch = ScrGetPitch;
  
  // Clear Display
  IniDisplay();

  // ini images 
  dx = ScrGetColumns;
  dy = 8;
  x  = 0;
  y  = (DispGetRows/3-dy)/2;
  ImageAssign(&EdgeAreaHor, ScrByteAddr(x, y), dx, dy, pitch);
  ImageAssign(&Area, OvlByteAddr(x, y), dx, dy, pitch);
  frameo(&Area);

  dx = ScrGetColumns;
  dy = DispGetRows/3;
  x  =    0;
  y  =   dy;
  ImageAssign(&Brightness, ScrByteAddr(x, y), dx, dy, pitch);
  ImageAssign(&Area, OvlByteAddr(x, y), dx, dy, pitch);
  frameo(&Area);

  dx = ScrGetColumns;
  dy = DispGetRows/3;
  x  =    0;
  y  = 2*dy;
  ImageAssign(&Difference, ScrByteAddr(x, y), dx, dy, pitch);
  ImageAssign(&Area, OvlByteAddr(x, y), dx, dy, pitch);
  frameo(&Area);


  do
  {
    tpict();

    ///////////////////////////////////////////////////////////////////////////////////////
    // Find Edge
    ///////////////////////////////////////////////////////////////////////////////////////

    VerticalEdgeDetection(&EdgeAreaHor, &Brightness, &Difference);

    ///////////////////////////////////////////////////////////////////////////////////////
    // Wait For Response
    ///////////////////////////////////////////////////////////////////////////////////////

    print("'q'=quite\n");

    response = rs232rcv();

  }
  while(response != 'q');

}

/************************************************************************************************************************/

I16 VerticalEdgeDetection (image *EdgeArea, image *Brightness, image *Difference)
{
  I16 dx, x, max, pos;
  U32 *pProj;

  dx = EdgeArea->dx;

  pProj  = (U32 *)sysmalloc( dx, MDATA);

  // projection of the edge
  projv(EdgeArea, pProj);

  DisplayGraph (Brightness, pProj, dx);

  for (x=0; x<dx-1; x++) 
  {
    pProj[x] = abs(pProj[x+1] - pProj[x]);
  }

  DisplayGraph (Difference, pProj, dx-1);
  

  max = pProj[0];
  pos = 0;
  for (x=1; x<dx-1; x++) 
  {
    if (pProj[x] > max)
    {
	  max = pProj[x];
	  pos = x;    
    }
  }

  return(pos);
}

/************************************************************************************************************************/

void IniDisplay (void)
{
  image Temp;

  ScrSetLogPage(ScrGetPhysPage);
  OvlSetLogPage(OvlGetPhysPage);

  // Clear Overlay
  ImageAssign(&Temp, OvlByteAddr(0, 0), DispGetColumns, DispGetRows, ScrGetPitch);
  set(&Temp, 0);  

  // Clear Image
  ImageAssign(&Temp, ScrByteAddr(0, 0), DispGetColumns, DispGetRows, ScrGetPitch);
  set(&Temp, 0);  

  set_ovlmask(255);

  vmode(vmOvlLive);
}

/************************************************************************************************************************/

void DisplayGraph (image *Graph, U32 *values, I16 nr)
{
  I32 i, x, y, dy, xold, yold;
  I16 max=0, scale=1, factor=1;
  

  dy    = Graph->dy;
  x     = 0;
  y     = 0;

  for (i=0; i<nr; i++) if (values[i] > max) {max=values[i];}

  if (max > dy) scale  = (max-1)/dy + 1;
  else           factor = dy/max;

  set(Graph, 0);
    
  for (i=0; i<nr; i++)
  {
    xold = x;
	yold = y;

    x = i;
    y = dy - ((factor * values[i])/scale);

    lined(Graph, xold, yold, x, y, 255);
  }
}

/************************************************************************************************************************/
