/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

This file reads the ID nr. and the camera type.

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

#define ID_FILE     "fd:/#ID.001"

/*******************************************************************/

int  GetCameraModel   (char *CameraModel);
int  GetCameraID      (int *CameraID);
int  ReadFileLine     (char *FileName, int line, int maxlength, char *text);

/*******************************************************************/

void main()
{
  char type[10];
  int  CamId;

  GetCameraModel(type);
  print("\nCamera typ = %s", type);

  GetCameraID(&CamId);
  print("\nCamera ID  = %d", CamId);
}

/*******************************************************************/

int GetCameraModel (char *CameraModel)

{
  int  res, error=0, line;
  char text[80], *ptext;

  line=0;
  res = ReadFileLine (ID_FILE, line, 80, text);
  
  if (res>=0)
  {
    ptext = text;
  
    while(*ptext != 'V') ptext++;
    while(*ptext) *CameraModel++ = *ptext++;
    *CameraModel=0;
  }
  else
  {
    error=-1;
  }
  
  return(error);
}

/*******************************************************************/

int GetCameraID (int *CameraID)

{
  int  res, error=0, nr=0, line;
  char text[80], *ptext;

  line=1;
  res = ReadFileLine (ID_FILE, line, 80, text);
  
  if (res>=0)
  {
    ptext = text;
  
    while(*ptext)
    {
      if (isdigit(*ptext)) nr = nr * 10 + (*ptext & 0x0F);
      ptext++;
    }
    
    *CameraID = nr;
  }
  else
  {
    error=-1;
  }
  
  return(error);
}

/*******************************************************************/

int ReadFileLine (char *FileName, int line, int maxlength, char *text)

{
  int  i, res, error=0;
  FILE *fp;

  /* read datas from file */
  fp = io_fopen(FileName, "r");

  if(fp)
  {
    // skip lines
    for (i=0; i<line; i++)
    {
      do
      {
        res=io_fgetc(fp);
        if (res == '\n') break;
      }
      while(res >= 0);
    }

    // read line
    for (i=0; i<maxlength-1; i++)
    {    
      res=io_fgetc(fp);

      if (res <     0) {error = -2; break;}
      if (res == '\n') break;
      
      *text++=(char)res;
    }

    // end of string
    *text=0;

    io_fclose(fp);
  }
  else
  {
    // File doesn't exist
    error = -1;
  }
  
  return(error);
}

/*******************************************************************/


